@@ -11,8 +11,11 @@
  * @brief Thread Declarations
  */
 
-#ifndef __GEARMAND_THREAD_H__
-#define __GEARMAND_THREAD_H__
+#pragma once
+
+#include <pthread.h>
+
+#include <libgearman-server/struct/gearmand_thread.h>
 
 #ifdef __cplusplus
 extern "C" {
@@ -27,28 +30,6 @@ extern "C" {
  * @{
  */
 
-struct gearmand_thread_st
-{
-  bool is_thread_lock;
-  bool is_wakeup_event;
-  uint32_t count;
-  uint32_t dcon_count;
-  uint32_t dcon_add_count;
-  uint32_t free_dcon_count;
-  int wakeup_fd[2];
-  gearmand_thread_st *next;
-  gearmand_thread_st *prev;
-  gearmand_st *gearmand;
-  struct event_base *base;
-  gearmand_con_st *dcon_list;
-  gearmand_con_st *dcon_add_list;
-  gearmand_con_st *free_dcon_list;
-  gearman_server_thread_st server_thread;
-  struct event wakeup_event;
-  pthread_t id;
-  pthread_mutex_t lock;
-};
-
 /**
  * Create a new gearmand thread.
  * @param gearmand Server instance structure previously initialized with
@@ -56,7 +37,7 @@ struct gearmand_thread_st
  * @return Standard gearman return value.
  */
 GEARMAN_API
-gearman_return_t gearmand_thread_create(gearmand_st *gearmand);
+gearmand_error_t gearmand_thread_create(struct gearmand_st *gearmand);
 
 /**
  * Free resources used by a thread.
@@ -88,5 +69,3 @@ void gearmand_thread_run(gearmand_thread_st *thread);
 #ifdef __cplusplus
 }
 #endif
-
-#endif /* __GEARMAND_THREAD_H__ */
